/**
	Definiciones para un programa
	para deformar un simple sistema resorte-masa-amortiguador en 2D
	(este programa usa primivitas de OpenGL y  Qt v. 4)

	Fraga  8/12/2010
	Fraga 28/01/2012
**/
#ifndef SPRING_H
#define SPRING_H

#include <QWidget>
#include <QtOpenGL>
#include <QTimer>
#include <QDoubleSpinBox>
#include <math.h>

class Spring: public QGLWidget
{
	Q_OBJECT  // Debe incluirse para usar seales/ranuras de Qt
public:
	Spring( QWidget *parent=0 );

	void createSpring( void );
	void muestraResorte( void );
	void Inicializa( void );
	QSizePolicy sizePolicy() const;

public slots:
	void resetea( void ) {
		timer->stop( );
		Inicializa( );

		updateGL();
	}
	void deforma( void ) { 
		r.rx.f = r.f * cos ( r.theta );
		r.ry.f = r.f * sin ( r.theta );

		r.rx.k3 = (r.rx.f * r.dt * r.dt)/r.m;
		r.ry.k3 = (r.ry.f * r.dt * r.dt)/r.m;
		timer->start( 33 );
	}
	void quitaFuerza( void ) { 
		r.f = 0.0;
		r.rx.f = r.ry.f = 0.0;
		r.rx.k3 = r.ry.k3 = 0.0;
	}
	void poneFuerza( double fuerza ) { 
		r.f = fuerza;
		// fprintf( stderr, "f = %lf\n", r.f );
	}
	void poneAngulo( double a ) { 
		r.theta = a * M_PI/180.0;
	}
	void unPasoDeformacion( void );

protected:
	void initializeGL( void );
	void resizeGL( int w, int h );
	void paintGL( void );
 
 
private:
	QTimer *timer;

	int windowWidth, windowHeight;   // canvas width and height

	float yrot, xrot;  // Encene rotation

	double angle;

	// Una estructura para almacenar el estado de un nodo.
	// Este es un sistema MRA con otro resorte extra solo con kint
	typedef struct mra {
		double f;
		double fi;
		double x_1, x, x1;
		double k3;
	} MRA;

	// Resorte en dos dimensiones
	typedef struct mra2d {
		double k, m, b;
		double dt;  // Delta i inicial: dt = 0.0333333333;
		double kint, dt2m; // El resorte interno
		double k1, k2;
		double f, theta;

		MRA rx, ry;	
	} MRA2D;

	MRA2D r; 
};


#endif // SPRING_H
