/** 
	A "hello" program in Qt
	that use OpenGL drawing primitives
	File: quad.cpp

	Fraga 19/09/2001
**/
#include <qpainter.h>
#include <ctype.h>
#include "spring.h"

Spring::Spring( QWidget *parent )
		: QGLWidget( parent )
{
	// setMouseTracking( TRUE );

	timer = new QTimer( this );
	connect ( timer, SIGNAL(timeout()), this, SLOT(unPasoDeformacion()) );


	setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );

	Inicializa( );
}

void Spring::Inicializa( void )
{
	// Para la estructura general
	r.f = 0.0;
	r.k = 11.11111;
	r.m = 1.0;
	r.b = 2.66667;

	r.dt = 0.033;
	r.theta = 0.0;

	r.k1 = 2.0 - r.dt*(r.k*r.dt + r.b)/r.m;
	r.k2 = 1.0 - r.dt*r.b/r.m;
	r.k3 = r.dt * r.dt / r.m; 
	r.k4 = r.k/2.0;

	// Para cada componente del resorte
	for( int i=0; i<3; i++ ) {
		r.rs[i].fx = r.rs[i].fy = 0.0; 
		r.rs[i].x_1 = r.rs[i].x = r.rs[i].x1 = 0.0;
		r.rs[i].y_1 = r.rs[i].y = r.rs[i].y1 = 0.0;
		r.rs[i].px = i+1.0;
		r.rs[i].py = 0.0;
	}

	p0x = p0y = 0.0;
	pnx = 4.0;
	pny = 0.0;

	n = 0;
}

//
// This method puts the initial coordinates each time window is resized
//
void Spring::resizeGL( int w, int h )
{
	float aspect=(float)w/(float)h;

	glViewport (0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();

	if( 2*w >= 3*h )
		glOrtho( -2.0*aspect + 2.0, 2.0*aspect + 2.0, -2.0, 2.0, -1.0, 1.0);
	else
		glOrtho( -1.0, 5.0, -3.0/aspect, 3.0/aspect, -1.0, 1.0);
}

void Spring::paintGL( void )
{
	/** Clear all pixels **/
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity ();			 /* clear the matrix */


	glLineWidth( 4.0 );
	glPointSize( 6.0 );
	muestraResortes(  );

	// swapBuffers( );
}

void Spring::initializeGL( ) 
{
	glClearColor ( 0.8, 0.8, 0.8, 0.0);	// Background to a grey tone

	/* initialize viewing values  */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(-1.0, 1.0, -1.0, 1.0, -1.0, 1.0);

	glEnable (GL_DEPTH_TEST);
}

void Spring::muestraResortes(  )
{
	int i;
	double px, py;

	glBegin( GL_LINES );
	glColor3f( 0.0, 0.0, 1.0);

	glVertex2d( p0x, 0.5 ); 
	glVertex2d( p0x,-0.5 ); 

	glVertex2d( pnx, 0.5 ); 
	glVertex2d( pnx,-0.5 ); 
	glEnd(  );


	glBegin( GL_LINE_STRIP );
	glColor3f( 0.0, 0.0, 1.0);

	glVertex2d( p0x, 0.0 ); 

	for ( i=0; i<3; i++){
		px = r.rs[i].px +  r.rs[i].x1;
		py = r.rs[i].py +  r.rs[i].y1;
		glVertex2d( px, py ); 
	}

	glVertex2d( pnx, pny ); 
	glEnd(  );

	glBegin( GL_POINTS );
	for ( i=0; i<3; i++ ){ 
		if ( i == n ) glColor3f( 1.0, 0.0, 0.0);
		else glColor3f( 0.0, 0.0, 1.0);
		px = r.rs[i].px +  r.rs[i].x1;
		py = r.rs[i].py +  r.rs[i].y1;
		glVertex3d( px, py, 0.1 ); 
	}

	glEnd(  );
}

void Spring::unPasoDeformacion( void )
{
	int i;
	MRA *pr;
	
	for( i=0; i<3; i++ ) {
		pr = &r.rs[i];

		if ( i != n ) {
			if ( i == 0 ) {
				pr->fx = r.rs[1].x1*r.k4;
				pr->fy = r.rs[1].y1*r.k4;
			}
			else if ( i == 2 ) {
				pr->fx = r.rs[i-1].x1*r.k4;
				pr->fy = r.rs[i-1].y1*r.k4;
			}
			else {
				pr->fx = (r.rs[i-1].x1 + r.rs[i+1].x1)*r.k4;
				pr->fy = (r.rs[i-1].y1 + r.rs[i+1].y1)*r.k4;
			}
		}
		
		pr->x1 = r.k1 * pr->x - r.k2 * pr->x_1 + r.k3 * pr->fx;
	    pr->x_1 = pr->x;
	    pr->x   = pr->x1;

	    pr->y1 = r.k1 * pr->y - r.k2 * pr->y_1 + r.k3 * pr->fy;
	    pr->y_1 = pr->y;
	    pr->y   = pr->y1;
	}
	

	updateGL();
}
	
